/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "scope/store/preview.h"
#include "scope/store/service_manager.h"
#include <unity/UnityExceptions.h>
#include <unity/scopes/ColumnLayout.h>
#include <unity/scopes/PreviewReply.h>
#include <unity/scopes/Variant.h>
#include <unity/scopes/VariantBuilder.h>
#include <unity/scopes/ActionMetadata.h>
#include <unity/scopes/Result.h>
#include <QDebug>


namespace usc = unity::scopes;


namespace
{
static usc::PreviewWidgetList
headerWidgets(Libertine::Store::Package const& app)
{
  usc::PreviewWidget header("hdr", "header");
  header.add_attribute_value("title", usc::Variant(app.name));
  header.add_attribute_value("subtitle", usc::Variant(app.summary));
  header.add_attribute_value("mascot", usc::Variant(app.icon));
  header.add_attribute_value("fallback", usc::Variant("image://theme/placeholder-app-icon"));

  usc::VariantBuilder builder;
  builder.add_tuple({{"value",  usc::Variant(app.render_rating())}});
  header.add_attribute_value("attributes", builder.end());

  usc::PreviewWidgetList widgets;
  widgets.push_back(header);

  qDebug() << "Pushed widgets for package:" << QString::fromStdString(app.name);
  return widgets;
}


static usc::PreviewWidget
metadataWidgets(Libertine::Store::Package const& app)
{
    usc::PreviewWidget widget("other_metadata", "table");
    usc::VariantArray values {
        usc::Variant{usc::VariantArray{usc::Variant{"Publisher/Creator"}, usc::Variant{app.publisher}}},
        usc::Variant{usc::VariantArray{usc::Variant{"Website"}, usc::Variant{app.website}}},
        usc::Variant{usc::VariantArray{usc::Variant{"License"}, usc::Variant{app.license}}},
    };
    widget.add_attribute_value("values", usc::Variant(values));
    return widget;
}
}


Libertine::Store::Preview::
Preview(usc::Result const&         result,
        usc::ActionMetadata const& metadata,
        std::shared_ptr<ServiceManager>  const& service)
  : PreviewQueryBase(result, metadata)
  , service_(service)
{
}


void Libertine::Store::Preview::
cancelled()
{
}


void Libertine::Store::Preview::
run(usc::PreviewReplyProxy const& reply)
{
  auto app = service_->app_info(QString::fromStdString(result()["id"].get_string()));

  auto widgets = headerWidgets(app);
  oneColumn.column.push_back("hdr");
  twoColumns.column1.push_back("hdr");

  widgets.push_back(metadataWidgets(app));
  oneColumn.column.push_back("other_metadata");
  twoColumns.column2.push_back("other_metadata");
  registerLayouts(reply);
  reply->push(widgets);
}


void Libertine::Store::Preview::
registerLayouts(unity::scopes::PreviewReplyProxy const& reply)
{
    usc::ColumnLayout layout1col(1);
    layout1col.add_column(oneColumn.column);

    usc::ColumnLayout layout2col(2);
    layout2col.add_column(twoColumns.column1);
    layout2col.add_column(twoColumns.column2);

    try
    {
        reply->register_layout({layout1col, layout2col});
    }
    catch (unity::LogicException const& e)
    {
        qWarning() << "Failed to register layout:" << QString::fromStdString(e.what());
    }
}
