/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scope/store/preview.h"
#include "tests/scope/store/mock_service_manager.h"

#include <unity/scopes/ActionMetadata.h>
#include <unity/scopes/PreviewWidget.h>
#include <unity/scopes/testing/MockPreviewReply.h>
#include <unity/scopes/testing/Result.h>
#include <gtest/gtest.h>
#include <gmock/gmock.h>

namespace
{
using namespace Libertine::Store;


TEST(StorePreviewTest, ShowsDetailedInformationForPackage)
{
  unity::scopes::testing::Result result;
  result["id"] = "vim.desktop";
  unity::scopes::ActionMetadata metadata("en_US", "phone");

  std::unique_ptr<unity::scopes::PreviewWidgetList> list(new unity::scopes::PreviewWidgetList());
  testing::NiceMock<unity::scopes::testing::MockPreviewReply> reply;
  EXPECT_CALL(reply, push(testing::_)).WillOnce(testing::SaveArg<0>(list.get()));
  unity::scopes::PreviewReplyProxy proxy(&reply, [](unity::scopes::PreviewReply*) {});

  std::shared_ptr<testing::NiceMock<MockServiceManager> > service(new testing::NiceMock<MockServiceManager>());
  Package package;
  package.name = "vim";
  package.summary = "classic text editor";
  package.icon = "file:///test/file.png";
  package.rating = 4.2;
  package.publisher = "Woody Boyd";
  package.website = "http://canonical.com/";
  package.license = "GPLv8+";

  EXPECT_CALL(*service, app_info(QString("vim.desktop"))).WillOnce(testing::Return(package));

  Preview preview(result, metadata, service);
  preview.run(proxy);

  ASSERT_NE(nullptr, list);
  ASSERT_EQ(2, list->size());

  auto header = list->front();
  EXPECT_EQ("hdr", header.id());
  EXPECT_EQ("header", header.widget_type());

  EXPECT_EQ(package.name, header.attribute_values()["title"].get_string());
  EXPECT_EQ(package.summary, header.attribute_values()["subtitle"].get_string());
  EXPECT_EQ(package.icon, header.attribute_values()["mascot"].get_string());

  auto attributes = header.attribute_values()["attributes"].get_array();
  ASSERT_EQ(1, attributes.size());
  EXPECT_EQ(package.render_rating(), attributes[0].get_dict()["value"].get_string());

  list->pop_front();
  auto props = list->front();
  EXPECT_EQ("other_metadata", props.id());
  EXPECT_EQ("table", props.widget_type());

  auto propValues = props.attribute_values()["values"].get_array();
  ASSERT_EQ(3, propValues.size());
  ASSERT_EQ(2, propValues[0].get_array().size());
  EXPECT_EQ("Publisher/Creator", propValues[0].get_array()[0].get_string());
  EXPECT_EQ(package.publisher, propValues[0].get_array()[1].get_string());
  ASSERT_EQ(2, propValues[1].get_array().size());
  EXPECT_EQ("Website", propValues[1].get_array()[0].get_string());
  EXPECT_EQ(package.website, propValues[1].get_array()[1].get_string());
  ASSERT_EQ(2, propValues[2].get_array().size());
  EXPECT_EQ("License", propValues[2].get_array()[0].get_string());
  EXPECT_EQ(package.license, propValues[2].get_array()[1].get_string());
}
}
